module modul_studenta (
    input  logic        rst,
    input  logic        clk,

    output logic        s_axil_awready,
    input  wire         s_axil_awvalid,
    input  wire [20:0]  s_axil_awaddr,
    input  wire [2:0]   s_axil_awprot,
    output logic        s_axil_wready,
    input  wire         s_axil_wvalid,
    input  wire [31:0]  s_axil_wdata,
    input  wire [3:0]   s_axil_wstrb,
    input  wire         s_axil_bready,
    output logic        s_axil_bvalid,
    output logic [1:0]  s_axil_bresp,
    output logic        s_axil_arready,
    input  wire         s_axil_arvalid,
    input  wire [20:0]  s_axil_araddr,
    input  wire [2:0]   s_axil_arprot,
    input  wire         s_axil_rready,
    output logic        s_axil_rvalid,
    output logic [31:0] s_axil_rdata,
    output logic [1:0]  s_axil_rresp,

    output logic [7:0]  LED
);

import registers_pkg::*;

registers_pkg::registers__out_t hwif_out;

logic [24:0] fsm_counter;
logic        fsm_pulse_reg;
logic        fsm_pulse;
logic        fsm_pulse_sig;
logic        fsm_enable_prev;

//===========================================
// FSM counter
//==========================================
always_ff @(posedge clk or posedge rst)
begin
    if (rst == 1'b1) begin
        fsm_enable_prev <= '0;
    end else begin
        fsm_enable_prev <= hwif_out.FSM_REGS.FSM_CONTROL.enable.value == 1'b1;
    end
end

always_ff @(posedge clk or posedge rst)
begin
    if (rst == 1'b1) begin
        fsm_counter <= '0;
    end else if (fsm_enable_prev == 1'b0 && hwif_out.FSM_REGS.FSM_CONTROL.enable.value == 1'b1) begin
        fsm_counter <= 'b0;
    end else begin
        fsm_counter <= fsm_counter + 1'b1;
    end
end

`ifdef SIMULATION
    assign fsm_pulse_sig = fsm_counter[3];
`else
    assign fsm_pulse_sig = fsm_counter[23];
`endif

always_ff @(posedge clk or posedge rst)
begin
    if (rst == 1'b1) begin
        fsm_pulse_reg <= '0;
    end else begin
        fsm_pulse_reg <= fsm_pulse_sig;
    end
end

assign fsm_pulse = fsm_pulse_sig & ~fsm_pulse_reg;

//==================================================
typedef enum logic [2:0] {
    IDLE    = 3'h0,
    LED_0   = 3'h1,
    LED_1   = 3'h2,
    LED_2   = 3'h3,
    LED_3   = 3'h4
} fsm_t;

fsm_t fsm , fsm_next;

//=======================
// FSM Register
//=======================
always_ff @(posedge clk or posedge rst)
begin
    if (rst == 1'b1) begin
        fsm <= IDLE;
    end else begin
        fsm <= fsm_next;
    end
end
//=========================

//=========================
// FSM state logic
//=========================
always_comb
begin

    case (fsm)
        IDLE   : begin
            if (fsm_pulse && hwif_out.FSM_REGS.FSM_CONTROL.enable.value == 1'b1) begin
                fsm_next = LED_0;
            end else begin
                fsm_next = IDLE;
            end
        end
        //----------------------------
        LED_0   : begin
            if (fsm_pulse)
                fsm_next = LED_1;
            else
                fsm_next = LED_0;
        end
        //----------------------------
        LED_1   : begin
            if (fsm_pulse)
                fsm_next = LED_2;
            else
                fsm_next = LED_1;
        end
        //----------------------------
        LED_2   : begin
            if (fsm_pulse)
                fsm_next = LED_3;
            else
                fsm_next = LED_2;
        end
        //----------------------------
        LED_3   : begin
            if (fsm_pulse) begin
                if (hwif_out.FSM_REGS.FSM_CONTROL.enable.value == 1'b1) begin
                    fsm_next = LED_0;
                end else begin 
                    fsm_next = IDLE;
                end
            end else begin
                fsm_next = LED_3;
            end
        end
        //----------------------------
        default: begin
            fsm_next = IDLE;
        end
    endcase
end

//==============================
// FSM Output logic
//=============================
always_comb
begin
    case (fsm)
        IDLE   : begin
            LED = 8'b1111_0000;
        end
        //----------------------------
        LED_0   : begin
            LED = 8'b0000_0001;
        end
        //----------------------------
        LED_1   : begin
            LED = 8'b0000_0010;
        end
        //----------------------------
        LED_2   : begin
            LED = 8'b0000_0100;
        end
        //----------------------------
        LED_3   : begin
            LED = 8'b0000_1000;
        end
        //----------------------------
        default: begin
            LED = 8'b0000_0000;
        end
    endcase
end

//assign LED = hwif_out.LEDS.LED.value;

//------------------------------------------
//------------- Registers ------------------
//------------------------------------------
registers u_registers (
    .clk                (clk),
    .rst                (rst),

    .s_axil_awready     (s_axil_awready),
    .s_axil_awvalid     (s_axil_awvalid),
    .s_axil_awaddr      (s_axil_awaddr[REGISTERS_MIN_ADDR_WIDTH-1:0]),
    .s_axil_awprot      (s_axil_awprot),

    .s_axil_wready      (s_axil_wready),
    .s_axil_wvalid      (s_axil_wvalid),
    .s_axil_wdata       (s_axil_wdata),
    .s_axil_wstrb       (s_axil_wstrb),

    .s_axil_bready      (s_axil_bready),
    .s_axil_bvalid      (s_axil_bvalid),
    .s_axil_bresp       (s_axil_bresp),

    .s_axil_arready     (s_axil_arready),
    .s_axil_arvalid     (s_axil_arvalid),
    .s_axil_araddr      (s_axil_araddr[REGISTERS_MIN_ADDR_WIDTH-1:0]),
    .s_axil_arprot      (s_axil_arprot),

    .s_axil_rready      (s_axil_rready),
    .s_axil_rvalid      (s_axil_rvalid),
    .s_axil_rdata       (s_axil_rdata),
    .s_axil_rresp       (s_axil_rresp),

    .hwif_out           (hwif_out)
);

endmodule